function fcwt1d(t,x)
% =========================================================================
% fcwt1d: Function of Continuous Wavelet Transform for 1-Dimension data 
%                                                   Sheng-An @ 2017
% =========================================================================
% Ref: Torrence, C., & Compo, G. P. (1998). A practical guide to wavelet 
%      analysis. Bulletin of the American Meteorological society, 79(1), 
%      61-78.
%
% All the formula used in this script can be found in that paper. 
% -------------------------------------------------------------------------

% Time spacing 
dt=t(2)-t(1);

% Nondimentional frequency taken to be 6 to satisfy the admissibility 
wo=6;

% The period (i.e., wavelenght) used for wavelets 
pmin = 4*dt;
pmax = abs(t(end)-t(1))/2;
lamda = logspace(log10(pmin),log10(pmax),50);

% The wavelet transform function 
wt=zeros(length(lamda),length(x));

for i=1:length(lamda)
l=lamda(i);

% The relation btw scale factor and the corresponding wavelength 
% lamda(i) = 4*pi*s/(wo+sqrt(2+wo^2));
s = l*(wo+sqrt(2+wo^2))/(4*pi);


% nt = 5*l;
% t1 = -nt/2:dt:nt/2;
t1 = t-mean(t);
tp = t1/s;

% Normalized Morlet wavelet (real part only)  
phi = sqrt(dt/s) * (pi)^(-1/4) * cos(wo*tp) .* exp(-tp.^2/2);

% figure; plot(t1, phi);
% tmp = cos(2*pi/lamda(i)*t1);
% hold on; plot(t1, tmp);
% stitle = sprintf('Lamda=%5.2f',lamda(i));
% title(stitle);

% Convolution 
wt(i,:) = conv(x,phi,'same');

end

figure; 

% Input data
subplot(211); 
plot(t,x); 
xlabel('Time'); ylabel('Signal'); title('Input data')

% Wavelet power spectrum 
subplot(212); 
[a,b]=meshgrid(t,lamda);
surf(a,b,wt); shading interp; view(0,90); 
set(gca,'ydir','reverse')
axis tight; 
xlabel('Time'); ylabel('Period'); title('Time-Frequency Spectrum')
colormap jet;
